@@ -286,19 +286,3 @@ class DirectedGraphHandler:
             h.remove_edge(*edge)
             result.append(edge)
         return result
-
-    @staticmethod
-    def export_to_graphml(g, path):
-        """Export given graph to a path in GraphML format.
-
-        Args:
-            g (DiGraph): Graph to export
-            path (str): Full output path for GraphML file
-
-        Returns:
-            bool: Operation success status
-        """
-        if not nx.is_directed_acyclic_graph(g):
-            return False
-        nx.write_graphml(g, path, prettyprint=True)
-        return True
