@@ -253,7 +253,9 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         self.connect_signals()
         self.read_settings()
         self.read_project_settings()
-        self._update_python_widgets_enabled(True if self.ui.radioButton_use_python_jupyter_console.isChecked() else False)
+        self._update_python_widgets_enabled(
+            True if self.ui.radioButton_use_python_jupyter_console.isChecked() else False
+        )
         self._update_julia_widgets_enabled(True if self.ui.radioButton_use_julia_jupyter_console.isChecked() else False)
 
     def connect_signals(self):
